@@ -41,10 +41,11 @@
 #include "config.h"
 #endif
 
-#include "clutter-color.h"
+#include "clutter-actor-private.h"
 #include "clutter-clone.h"
 #include "clutter-debug.h"
 #include "clutter-main.h"
+#include "clutter-paint-volume-private.h"
 #include "clutter-private.h"
 
 #include "cogl/cogl.h"
@@ -133,6 +134,10 @@ clutter_clone_apply_transform (ClutterActor *self, CoglMatrix *matrix)
   CLUTTER_ACTOR_CLASS (clutter_clone_parent_class)->apply_transform (self,
                                                                      matrix);
 
+  /* if we don't have a source, nothing else to do */
+  if (priv->clone_source == NULL)
+    return;
+
   /* get our allocated size */
   clutter_actor_get_allocation_geometry (self, &geom);
 
@@ -155,7 +160,7 @@ clutter_clone_paint (ClutterActor *self)
   ClutterClonePrivate *priv = clone->priv;
   gboolean was_unmapped = FALSE;
 
-  if (G_UNLIKELY (priv->clone_source == NULL))
+  if (priv->clone_source == NULL)
     return;
 
   CLUTTER_NOTE (PAINT,
@@ -179,7 +184,9 @@ clutter_clone_paint (ClutterActor *self)
       was_unmapped = TRUE;
     }
 
+  _clutter_actor_push_clone_paint ();
   clutter_actor_paint (priv->clone_source);
+  _clutter_actor_pop_clone_paint ();
 
   if (was_unmapped)
     _clutter_actor_set_enable_paint_unmapped (priv->clone_source, FALSE);
@@ -195,6 +202,10 @@ clutter_clone_get_paint_volume (ClutterActor *self,
   ClutterClonePrivate *priv = CLUTTER_CLONE (self)->priv;
   const ClutterPaintVolume *source_volume;
 
+  /* if the source is not set the paint volume is defined to be empty */
+  if (priv->clone_source == NULL)
+    return TRUE;
+
   /* query the volume of the source actor and simply masquarade it as
    * the clones volume... */
   source_volume = clutter_actor_get_paint_volume (priv->clone_source);
@@ -218,7 +229,7 @@ clutter_clone_allocate (ClutterActor           *self,
   parent_class = CLUTTER_ACTOR_CLASS (clutter_clone_parent_class);
   parent_class->allocate (self, box, flags);
 
-  if (G_UNLIKELY (priv->clone_source == NULL))
+  if (priv->clone_source == NULL)
     return;
 
   /* we act like a "foster parent" for the source we are cloning;
@@ -285,7 +296,6 @@ clutter_clone_class_init (ClutterCloneClass *klass)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   ClutterActorClass *actor_class = CLUTTER_ACTOR_CLASS (klass);
-  GParamSpec *pspec = NULL;
 
   g_type_class_add_private (gobject_class, sizeof (ClutterClonePrivate));
 
@@ -307,14 +317,17 @@ clutter_clone_class_init (ClutterCloneClass *klass)
    *
    * Since: 1.0
    */
-  pspec = g_param_spec_object ("source",
-                               P_("Source"),
-                               P_("Specifies the actor to be cloned"),
-                               CLUTTER_TYPE_ACTOR,
-                               G_PARAM_CONSTRUCT |
-                               CLUTTER_PARAM_READWRITE);
-  obj_props[PROP_SOURCE] = pspec;
-  g_object_class_install_property (gobject_class, PROP_SOURCE, pspec);
+  obj_props[PROP_SOURCE] =
+    g_param_spec_object ("source",
+                         P_("Source"),
+                         P_("Specifies the actor to be cloned"),
+                         CLUTTER_TYPE_ACTOR,
+                         G_PARAM_CONSTRUCT |
+                         CLUTTER_PARAM_READWRITE);
+
+  _clutter_object_class_install_properties (gobject_class,
+                                            PROP_LAST,
+                                            obj_props);
 }
 
 static void
@@ -362,12 +375,7 @@ static void
 clutter_clone_set_source_internal (ClutterClone *clone,
 				   ClutterActor *source)
 {
-  ClutterClonePrivate *priv;
-
-  g_return_if_fail (CLUTTER_IS_CLONE (clone));
-  g_return_if_fail (source == NULL || CLUTTER_IS_ACTOR (source));
-
-  priv = clone->priv;
+  ClutterClonePrivate *priv = clone->priv;
 
   if (priv->clone_source)
     {
