@@ -135,6 +135,8 @@ struct _ClutterStagePrivate
 
   ClutterPickMode     pick_buffer_mode;
 
+  GHashTable *devices;
+
   guint relayout_pending       : 1;
   guint redraw_pending         : 1;
   guint is_fullscreen          : 1;
@@ -1202,10 +1204,13 @@ clutter_stage_finalize (GObject *object)
   g_queue_foreach (priv->event_queue, (GFunc)clutter_event_free, NULL);
   g_queue_free (priv->event_queue);
 
-  g_free (stage->priv->title);
+  g_free (priv->title);
 
   g_array_free (priv->paint_volume_stack, TRUE);
 
+  if (priv->devices != NULL)
+    g_hash_table_destroy (priv->devices);
+
   G_OBJECT_CLASS (clutter_stage_parent_class)->finalize (object);
 }
 
@@ -1690,7 +1695,7 @@ clutter_stage_set_color (ClutterStage       *stage,
 /**
  * clutter_stage_get_color:
  * @stage: A #ClutterStage
- * @color: return location for a #ClutterColor
+ * @color: (out): return location for a #ClutterColor
  *
  * Retrieves the stage color.
  */
@@ -2783,18 +2788,20 @@ clutter_stage_queue_redraw (ClutterStage *stage)
 gboolean
 clutter_stage_is_default (ClutterStage *stage)
 {
+  ClutterStageManager *stage_manager;
   ClutterStageWindow *impl;
 
   g_return_val_if_fail (CLUTTER_IS_STAGE (stage), FALSE);
 
-  if (CLUTTER_ACTOR (stage) == clutter_stage_get_default ())
-    return TRUE;
+  stage_manager = clutter_stage_manager_get_default ();
+  if (stage != clutter_stage_manager_get_default_stage (stage_manager))
+    return FALSE;
 
   impl = _clutter_stage_get_window (stage);
-  if (impl == _clutter_stage_get_default_window ())
-    return TRUE;
+  if (impl != _clutter_stage_get_default_window ())
+    return FALSE;
 
-  return FALSE;
+  return TRUE;
 }
 
 void
@@ -3325,7 +3332,7 @@ _clutter_stage_maybe_finish_queue_redraws (ClutterStage *stage)
 }
 
 /**
- * clutter_stage_get_accept_focus:
+ * clutter_stage_set_accept_focus:
  * @stage: a #ClutterStage
  * @accept_focus: %TRUE to accept focus on show
  *
@@ -3373,3 +3380,44 @@ clutter_stage_get_accept_focus (ClutterStage *stage)
 
   return stage->priv->accept_focus;
 }
+
+void
+_clutter_stage_add_device (ClutterStage       *stage,
+                           ClutterInputDevice *device)
+{
+  ClutterStagePrivate *priv = stage->priv;
+
+  if (G_UNLIKELY (priv->devices == NULL))
+    priv->devices = g_hash_table_new (NULL, NULL);
+
+  if (g_hash_table_lookup (priv->devices, device) != NULL)
+    return;
+
+  g_hash_table_insert (priv->devices, device, GINT_TO_POINTER (1));
+  _clutter_input_device_set_stage (device, stage);
+}
+
+void
+_clutter_stage_remove_device (ClutterStage       *stage,
+                              ClutterInputDevice *device)
+{
+  ClutterStagePrivate *priv = stage->priv;
+
+  if (G_UNLIKELY (priv->devices == NULL))
+    return;
+
+  _clutter_input_device_set_stage (device, NULL);
+  g_hash_table_remove (priv->devices, device);
+}
+
+gboolean
+_clutter_stage_has_device (ClutterStage       *stage,
+                           ClutterInputDevice *device)
+{
+  ClutterStagePrivate *priv = stage->priv;
+
+  if (priv->devices == NULL)
+    return FALSE;
+
+  return g_hash_table_lookup (priv->devices, device) != NULL;
+}
