@@ -100,6 +100,9 @@ typedef struct _ClutterStateKey
   GValue           value;        /* target value */
   gdouble          pre_delay;    /* fraction of duration to delay before
                                     starting */
+  gdouble          pre_pre_delay;/* fraction of duration to add to
+                                    pre_delay. This is used to set keys
+                                    during transitions. */
   gdouble          post_delay;   /* fraction of duration to be done in */
 
   State           *source_state; /* source state */
@@ -302,6 +305,10 @@ again_for_target_state:
                   /* no more keys with transitions to this target_state*/
                   if (target_state->keys == NULL)
                     {
+                      /* If this state is the current state, unset the state */
+                      if (target_state == this->priv->target_state)
+                        clutter_state_set_state (this, NULL);
+
                       /* remove any keys that exist that uses this state as a source */
                       clutter_state_remove_key (this, s->data, NULL, NULL, NULL);
 
@@ -429,9 +436,10 @@ clutter_state_new_frame (ClutterTimeline *timeline,
           if (found_specific || key->source_state == NULL)
             {
               const GValue *value;
+              gdouble pre_delay = key->pre_delay + key->pre_pre_delay;
 
-              sub_progress = (progress - key->pre_delay) /
-                             (1.0 - (key->pre_delay + key->post_delay));
+              sub_progress = (progress - pre_delay) /
+                             (1.0 - (pre_delay + key->post_delay));
 
               if (sub_progress >= 0.0)
                 {
@@ -468,17 +476,43 @@ clutter_state_change (ClutterState *state,
   GList               *k;
 
   g_return_val_if_fail (CLUTTER_IS_STATE (state), NULL);
-  g_return_val_if_fail (target_state_name != NULL, NULL);
 
   priv = state->priv;
 
-  if (target_state_name != NULL)
-    target_state_name = g_intern_string (target_state_name);
+  /* If we've been asked to change state to NULL, reset the
+   * ClutterState to its initial state, but leave the keys
+   * alone.
+   */
+  if (!target_state_name)
+    {
+      if (!priv->target_state)
+        return NULL;
+
+      priv->source_state_name = priv->target_state_name = NULL;
+      priv->source_state = priv->target_state = NULL;
+
+      clutter_timeline_stop (priv->timeline);
+      clutter_timeline_rewind (priv->timeline);
+
+      if (priv->current_animator)
+        {
+          clutter_animator_set_timeline (priv->current_animator, NULL);
+          priv->current_animator = NULL;
+        }
+
+      return NULL;
+    }
+
+  target_state_name = g_intern_string (target_state_name);
 
   if (target_state_name == priv->target_state_name)
     {
-      /* Avoid transitioning if the desired state is already current */
-      return priv->timeline;
+      /* Avoid transitioning if the desired state is already current,
+       * unless we're warping to it and the state transition is in
+       * progress (in that case, immediately warp to the state).
+       */
+      if (!clutter_timeline_is_playing (priv->timeline) || animate)
+        return priv->timeline;
     }
 
   if (priv->current_animator != NULL)
@@ -527,6 +561,11 @@ clutter_state_change (ClutterState *state,
           ClutterStateKey *key = k->data;
           GValue initial = { 0, };
 
+          /* Reset the pre-pre-delay - this is only used for setting keys
+           * during transitions.
+           */
+          key->pre_pre_delay = 0;
+
           g_value_init (&initial, clutter_interval_get_value_type (key->interval));
           g_object_get_property (key->object, key->property_name, &initial);
 
@@ -567,11 +606,14 @@ clutter_state_change (ClutterState *state,
  * @state: a #ClutterState
  * @target_state_name: the state to transition to
  *
- * Change the current state of #ClutterState to @target_state_name
+ * Change the current state of #ClutterState to @target_state_name.
  *
  * The state will animate during its transition, see
  * #clutter_state_warp_to_state for animation-free state switching.
  *
+ * Setting a %NULL state will stop the current animation and unset
+ * the current state, but keys will be left intact.
+ *
  * Return value: (transfer none): the #ClutterTimeline that drives the
  *   state transition. The returned timeline is owned by the #ClutterState
  *   and it should not be unreferenced
@@ -590,10 +632,10 @@ clutter_state_set_state (ClutterState *state,
  * @state: a #ClutterState
  * @target_state_name: the state to transition to
  *
- * Change the current state of #ClutterState to @target_state_name
- *
  * Change to the specified target state immediately with no animation.
  *
+ * See clutter_state_set_state().
+ *
  * Return value: (transfer none): the #ClutterTimeline that drives the
  *   state transition. The returned timeline is owned by the #ClutterState
  *   and it should not be unreferenced
@@ -677,9 +719,9 @@ get_property_from_object (GObject     *gobject,
  *
  * |[
  *   clutter_state_set (state, NULL, "hover",
- *                      button, "opacity", 255, CLUTTER_LINEAR,
- *                      button, "scale-x", 1.2, CLUTTER_EASE_OUT_CUBIC,
- *                      button, "scale-y", 1.2, CLUTTER_EASE_OUT_CUBIC,
+ *                      button, "opacity", CLUTTER_LINEAR, 255,
+ *                      button, "scale-x", CLUTTER_EASE_OUT_CUBIC, 1.2,
+ *                      button, "scale-y", CLUTTER_EASE_OUT_CUBIC, 1.2,
  *                      NULL);
  * ]|
  *
@@ -690,10 +732,10 @@ get_property_from_object (GObject     *gobject,
  * mode, and the #ClutterActor:scale-x and #ClutterActor:scale-y properties
  * animated to a value of 1.2 using %CLUTTER_EASE_OUT_CUBIC as the animation
  * mode. To change the state (and start the transition) you can use the
- * clutter_state_change() function:
+ * clutter_state_set_state() function:
  *
  * |[
- *   clutter_state_change (state, "hover", TRUE);
+ *   clutter_state_set_state (state, "hover");
  * ]|
  *
  * If a given object, state_name, property tuple already exist in the
@@ -707,8 +749,8 @@ get_property_from_object (GObject     *gobject,
  *
  * |[
  *   clutter_state_set (state, "hover", "toggled",
- *                      button, "delayed::scale-x", 1.0, 0.2, 0.2,
- *                      button, "delayed::scale-y", 1.0, 0.2, 0.2,
+ *                      button, "delayed::scale-x", CLUTTER_LINEAR, 1.0, 0.2, 0.2,
+ *                      button, "delayed::scale-y", CLUTTER_LINEAR, 1.0, 0.2, 0.2,
  *                      NULL);
  * ]|
  *
@@ -759,14 +801,9 @@ clutter_state_set (ClutterState *state,
       if (pspec == NULL)
         break;
 
-#if GLIB_CHECK_VERSION (2, 23, 2)
       G_VALUE_COLLECT_INIT (&value, G_PARAM_SPEC_VALUE_TYPE (pspec),
                             args, 0,
                             &error);
-#else
-      g_value_init (&value, G_PARAM_SPEC_VALUE_TYPE (pspec));
-      G_VALUE_COLLECT (&value, args, 0, &error);
-#endif /* GLIB_CHECK_VERSION */
 
       if (error != NULL)
         {
@@ -820,6 +857,7 @@ static void
 clutter_state_set_key_internal (ClutterState    *state,
                                 ClutterStateKey *key)
 {
+  ClutterStatePrivate *priv = state->priv;
   State *target_state = key->target_state;
   GList *old_item = NULL;
 
@@ -836,6 +874,50 @@ clutter_state_set_key_internal (ClutterState    *state,
   target_state->keys = g_list_insert_sorted (target_state->keys,
                                              key,
                                              sort_props_func);
+
+  /* If the current target state is modified, we have some work to do.
+   *
+   * If the animation is running, we add a key to the current animation
+   * with a delay of the current duration so that the new animation will
+   * animate into place.
+   *
+   * If the animation isn't running, but the state is set, we immediately
+   * warp to that state.
+   */
+  if (key->target_state == priv->target_state)
+    {
+      if (!clutter_timeline_is_playing (priv->timeline))
+        {
+          /* We can warp to the state by setting a NULL state, then setting
+           * the target state again.
+           */
+          clutter_state_change (state, NULL, FALSE);
+          clutter_state_change (state, target_state->name, FALSE);
+        }
+      else
+        {
+          /* Set the ClutterInterval associated with the state */
+          GValue initial = { 0, };
+          gdouble progress = clutter_timeline_get_progress (priv->timeline);
+
+          g_value_init (&initial,
+                        clutter_interval_get_value_type (key->interval));
+          g_object_get_property (key->object, key->property_name, &initial);
+
+          if (clutter_alpha_get_mode (key->alpha) != key->mode)
+            clutter_alpha_set_mode (key->alpha, key->mode);
+
+          clutter_interval_set_initial_value (key->interval, &initial);
+          clutter_interval_set_final_value (key->interval, &key->value);
+
+          g_value_unset (&initial);
+
+          /* Set the delay as if the interval had just begun */
+          if (progress > key->pre_delay)
+            key->pre_pre_delay = MIN (progress - key->pre_delay,
+                                      1.0 - key->post_delay);
+        }
+    }
 }
 
 /*
@@ -1160,7 +1242,8 @@ clutter_state_class_init (ClutterStateClass *klass)
    * @state: the #ClutterState that emitted the signal
    *
    * The ::completed signal is emitted when a #ClutterState reaches
-   * the target state specified by clutter_state_change()
+   * the target state specified by clutter_state_set_state() or
+   * clutter_state_warp_to_state().
    *
    * Since: 1.4
    */
@@ -1178,8 +1261,8 @@ clutter_state_class_init (ClutterStateClass *klass)
    *
    * The currently set target state, setting it causes the
    * state machine to transition to the new state, use
-   * clutter_state_change() with a final FALSE argument to
-   * change state without a transition.
+   * clutter_state_warp_to_state() to change state without
+   * a transition.
    */
   pspec = g_param_spec_string ("state",
                                P_("State"),
@@ -1360,18 +1443,9 @@ clutter_state_key_copy (gpointer boxed)
   return boxed;
 }
 
-GType
-clutter_state_key_get_type (void)
-{
-  static GType our_type = 0;
-
-  if (!our_type)
-    our_type = g_boxed_type_register_static (I_("ClutterStateKey"),
-                                             clutter_state_key_copy,
-                                             clutter_state_key_free);
-
-  return our_type;
-}
+G_DEFINE_BOXED_TYPE (ClutterStateKey, clutter_state_key,
+                     clutter_state_key_copy,
+                     clutter_state_key_free);
 
 /**
  * clutter_state_key_get_pre_delay:
