@@ -18,9 +18,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
  *
  * ClutterTimeoutPool: pool of timeout functions using the same slice of
  *                     the GLib main loop
@@ -34,8 +34,9 @@
 #include "config.h"
 #endif
 
-#include "clutter-debug.h"
 #include "clutter-timeout-pool.h"
+
+#include "clutter-debug.h"
 #include "clutter-timeout-interval.h"
 
 typedef struct _ClutterTimeout  ClutterTimeout;
@@ -63,8 +64,9 @@ struct _ClutterTimeoutPool
 
   guint next_id;
 
-  GTimeVal start_time;
-  GList *timeouts, *dispatched_timeouts;
+  GList *timeouts;
+  GList *dispatched_timeouts;
+
   gint ready;
 
   guint id;
@@ -133,11 +135,21 @@ clutter_timeout_prepare (ClutterTimeoutPool *pool,
                          ClutterTimeout     *timeout,
                          gint               *next_timeout)
 {
-  GTimeVal now;
+  GSource *source = (GSource *) pool;
+  gint64 now;
 
-  g_source_get_current_time (&pool->source, &now);
+#if GLIB_CHECK_VERSION (2, 27, 3)
+  now = g_source_get_time (source) / 1000;
+#else
+  {
+    GTimeVal source_time;
+    g_source_get_current_time (source, &source_time);
+    now = source_time.tv_sec * 1000 + source_time.tv_usec / 1000;
+  }
+#endif
 
-  return _clutter_timeout_interval_prepare (&now, &timeout->interval,
+  return _clutter_timeout_interval_prepare (now,
+                                            &timeout->interval,
                                             next_timeout);
 }
 
@@ -209,7 +221,7 @@ static gboolean
 clutter_timeout_pool_check (GSource *source)
 {
   ClutterTimeoutPool *pool = (ClutterTimeoutPool *) source;
-  GList *l = pool->timeouts;
+  GList *l;
 
   clutter_threads_enter ();
 
@@ -365,6 +377,8 @@ clutter_timeout_pool_finalize (GSource *source)
  *   the destruction of the timeout pool using g_source_destroy()
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.6
  */
 ClutterTimeoutPool *
 clutter_timeout_pool_new (gint priority)
@@ -377,12 +391,12 @@ clutter_timeout_pool_new (gint priority)
   if (!source)
     return NULL;
 
+  g_source_set_name (source, "Clutter timeout pool");
+
   if (priority != G_PRIORITY_DEFAULT)
     g_source_set_priority (source, priority);
 
   pool = (ClutterTimeoutPool *) source;
-
-  g_get_current_time (&pool->start_time);
   pool->next_id = 1;
   pool->id = g_source_attach (source, NULL);
 
@@ -419,6 +433,8 @@ clutter_timeout_pool_new (gint priority)
  *   Use clutter_timeout_pool_remove() to stop the timeout.
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.6
  */
 guint
 clutter_timeout_pool_add (ClutterTimeoutPool *pool,
@@ -454,6 +470,8 @@ clutter_timeout_pool_add (ClutterTimeoutPool *pool,
  * clutter_timeout_pool_add().
  *
  * Since: 0.4
+ *
+ * Deprecated: 1.6
  */
 void
 clutter_timeout_pool_remove (ClutterTimeoutPool *pool,
