@@ -154,15 +154,14 @@ cogl_buffer_get_update_hint (CoglBuffer *buffer);
 typedef enum { /*< prefix=COGL_BUFFER_ACCESS >*/
  COGL_BUFFER_ACCESS_READ       = 1 << 0,
  COGL_BUFFER_ACCESS_WRITE      = 1 << 1,
- COGL_BUFFER_ACCESS_READ_WRITE = COGL_BUFFER_ACCESS_READ |
-                                 COGL_BUFFER_ACCESS_WRITE
+ COGL_BUFFER_ACCESS_READ_WRITE = COGL_BUFFER_ACCESS_READ | COGL_BUFFER_ACCESS_WRITE
 } CoglBufferAccess;
 
 
 /**
  * CoglBufferMapHint:
- * COGL_BUFFER_MAP_HINT_DISCARD: Tells Cogl that you plan to replace
- *                               all the buffer's contents.
+ * @COGL_BUFFER_MAP_HINT_DISCARD: Tells Cogl that you plan to replace
+ *    all the buffer's contents.
  *
  * Hints to Cogl about how you are planning to modify the data once it
  * is mapped.
@@ -171,7 +170,7 @@ typedef enum { /*< prefix=COGL_BUFFER_ACCESS >*/
  * Stability: Unstable
  */
 typedef enum { /*< prefix=COGL_BUFFER_MAP_HINT >*/
-    COGL_BUFFER_MAP_HINT_DISCARD = 1 << 0
+  COGL_BUFFER_MAP_HINT_DISCARD = 1 << 0
 } CoglBufferMapHint;
 
 /**
@@ -179,7 +178,7 @@ typedef enum { /*< prefix=COGL_BUFFER_MAP_HINT >*/
  * @buffer: a buffer object
  * @access: how the mapped buffer will be used by the application
  * @hints: A mask of #CoglBufferMapHint<!-- -->s that tell Cogl how
- *        the data will be modified once mapped.
+ *   the data will be modified once mapped.
  *
  * Maps the buffer into the application address space for direct access.
  *
@@ -199,7 +198,7 @@ typedef enum { /*< prefix=COGL_BUFFER_MAP_HINT >*/
  * Since: 1.2
  * Stability: Unstable
  */
-guint8 *
+void *
 cogl_buffer_map (CoglBuffer        *buffer,
                  CoglBufferAccess   access,
                  CoglBufferMapHint  hints);
@@ -235,7 +234,7 @@ cogl_buffer_unmap (CoglBuffer *buffer);
 gboolean
 cogl_buffer_set_data (CoglBuffer  *buffer,
                       gsize        offset,
-                      const guint8 *data,
+                      const void  *data,
                       gsize        size);
 
 G_END_DECLS
