@@ -203,6 +203,14 @@ add_stencil_clip_rectangle (float x_1,
   /* temporarily swap in our special stenciling pipeline */
   cogl_push_source (ctx->stencil_pipeline);
 
+  /* This can be called from the journal code which doesn't flush
+     the matrix stacks between calls so we need to ensure they're
+     flushed now */
+  _cogl_matrix_stack_flush_to_gl (modelview_stack,
+                                  COGL_MATRIX_MODELVIEW);
+  _cogl_matrix_stack_flush_to_gl (projection_stack,
+                                  COGL_MATRIX_PROJECTION);
+
   if (first)
     {
       GE( glEnable (GL_STENCIL_TEST) );
@@ -215,14 +223,6 @@ add_stencil_clip_rectangle (float x_1,
       GE( glStencilFunc (GL_NEVER, 0x1, 0x1) );
       GE( glStencilOp (GL_REPLACE, GL_REPLACE, GL_REPLACE) );
 
-      /* This can be called from the journal code which doesn't flush
-         the matrix stacks between calls so we need to ensure they're
-         flushed now */
-      _cogl_matrix_stack_flush_to_gl (modelview_stack,
-                                      COGL_MATRIX_MODELVIEW);
-      _cogl_matrix_stack_flush_to_gl (projection_stack,
-                                      COGL_MATRIX_PROJECTION);
-
       _cogl_rectangle_immediate (x_1, y_1, x_2, y_2);
     }
   else
@@ -597,6 +597,8 @@ _cogl_clip_stack_flush (CoglClipStack *stack)
   /* If the stack is empty then there's nothing else to do */
   if (stack == NULL)
     {
+      COGL_NOTE (CLIPPING, "Flushed empty clip stack");
+
       ctx->current_clip_stack_uses_stencil = FALSE;
       GE (glDisable (GL_SCISSOR_TEST));
       return;
@@ -636,6 +638,10 @@ _cogl_clip_stack_flush (CoglClipStack *stack)
         }
     }
 
+  COGL_NOTE (CLIPPING, "Flushing scissor to (%i, %i, %i, %i)",
+             scissor_x0, scissor_y0,
+             scissor_x1, scissor_y1);
+
   GE (glEnable (GL_SCISSOR_TEST));
   GE (glScissor (scissor_x0, scissor_y_start,
                  scissor_x1 - scissor_x0,
@@ -651,6 +657,8 @@ _cogl_clip_stack_flush (CoglClipStack *stack)
         {
           CoglClipStackPath *path_entry = (CoglClipStackPath *) entry;
 
+          COGL_NOTE (CLIPPING, "Adding stencil clip for path");
+
           _cogl_matrix_stack_push (modelview_stack);
           _cogl_matrix_stack_set (modelview_stack, &path_entry->matrix);
 
@@ -677,6 +685,8 @@ _cogl_clip_stack_flush (CoglClipStack *stack)
                  them then use that instead */
               if (has_clip_planes)
                 {
+                  COGL_NOTE (CLIPPING, "Adding clip planes clip for rectangle");
+
                   set_clip_planes (rect->x0,
                                    rect->y0,
                                    rect->x1,
@@ -687,6 +697,8 @@ _cogl_clip_stack_flush (CoglClipStack *stack)
                 }
               else
                 {
+                  COGL_NOTE (CLIPPING, "Adding stencil clip for rectangle");
+
                   add_stencil_clip_rectangle (rect->x0,
                                               rect->y0,
                                               rect->x1,
