@@ -3,7 +3,7 @@
  *
  * An object oriented GL/GLES Abstraction/Utility Layer
  *
- * Copyright (C) 2009 Intel Corporation.
+ * Copyright (C) 2009,2010 Intel Corporation.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -34,6 +34,7 @@
 #include "cogl-internal.h"
 #include "cogl-matrix-stack.h"
 #include "cogl-framebuffer-private.h"
+#include "cogl-object-private.h"
 
 typedef struct {
   CoglMatrix matrix;
@@ -53,13 +54,21 @@ typedef struct {
  */
 struct _CoglMatrixStack
 {
+  CoglObject _parent;
+
   GArray *stack;
 
   /* which state does GL have, NULL if unknown */
   CoglMatrixState *flushed_state;
   gboolean flushed_identity;
+
+  unsigned int age;
 };
 
+static void _cogl_matrix_stack_free (CoglMatrixStack *stack);
+
+COGL_OBJECT_INTERNAL_DEFINE (MatrixStack, matrix_stack);
+
 /* XXX: this doesn't initialize the matrix! */
 static void
 _cogl_matrix_state_init (CoglMatrixState *state)
@@ -138,11 +147,13 @@ _cogl_matrix_stack_new (void)
   _cogl_matrix_state_init (state);
   state->is_identity = TRUE;
 
-  return stack;
+  stack->age = 0;
+
+  return _cogl_matrix_stack_object_new (stack);
 }
 
-void
-_cogl_matrix_stack_destroy (CoglMatrixStack *stack)
+static void
+_cogl_matrix_stack_free (CoglMatrixStack *stack)
 {
   g_array_free (stack->stack, TRUE);
   g_slice_free (CoglMatrixStack, stack);
@@ -185,6 +196,7 @@ _cogl_matrix_stack_pop (CoglMatrixStack *stack)
           stack->flushed_state = NULL;
         }
 
+      stack->age++;
       g_array_set_size (stack->stack, stack->stack->len - 1);
     }
 }
@@ -209,6 +221,7 @@ _cogl_matrix_stack_load_identity (CoglMatrixStack *stack)
 
       /* mark dirty */
       stack->flushed_state = NULL;
+      stack->age++;
     }
 }
 
@@ -225,6 +238,7 @@ _cogl_matrix_stack_scale (CoglMatrixStack *stack,
   /* mark dirty */
   stack->flushed_state = NULL;
   state->is_identity = FALSE;
+  stack->age++;
 }
 
 void
@@ -240,6 +254,7 @@ _cogl_matrix_stack_translate (CoglMatrixStack *stack,
   /* mark dirty */
   stack->flushed_state = NULL;
   state->is_identity = FALSE;
+  stack->age++;
 }
 
 void
@@ -256,6 +271,7 @@ _cogl_matrix_stack_rotate (CoglMatrixStack *stack,
   /* mark dirty */
   stack->flushed_state = NULL;
   state->is_identity = FALSE;
+  stack->age++;
 }
 
 void
@@ -269,6 +285,7 @@ _cogl_matrix_stack_multiply (CoglMatrixStack  *stack,
   /* mark dirty */
   stack->flushed_state = NULL;
   state->is_identity = FALSE;
+  stack->age++;
 }
 
 void
@@ -289,6 +306,7 @@ _cogl_matrix_stack_frustum (CoglMatrixStack *stack,
   /* mark dirty */
   stack->flushed_state = NULL;
   state->is_identity = FALSE;
+  stack->age++;
 }
 
 void
@@ -306,6 +324,7 @@ _cogl_matrix_stack_perspective (CoglMatrixStack *stack,
   /* mark dirty */
   stack->flushed_state = NULL;
   state->is_identity = FALSE;
+  stack->age++;
 }
 
 void
@@ -325,6 +344,7 @@ _cogl_matrix_stack_ortho (CoglMatrixStack *stack,
   /* mark dirty */
   stack->flushed_state = NULL;
   state->is_identity = FALSE;
+  stack->age++;
 }
 
 gboolean
@@ -370,6 +390,64 @@ _cogl_matrix_stack_set (CoglMatrixStack  *stack,
   /* mark dirty */
   stack->flushed_state = NULL;
   state->is_identity = FALSE;
+  stack->age++;
+}
+
+#ifndef HAVE_COGL_GLES2
+
+static void
+flush_to_fixed_api_gl (gboolean is_identity,
+                       const CoglMatrix *matrix,
+                       void *user_data)
+{
+  CoglMatrixStack *stack = user_data;
+
+  if (is_identity)
+    {
+      if (!stack->flushed_identity)
+        GE (glLoadIdentity ());
+      stack->flushed_identity = TRUE;
+    }
+  else
+    {
+      GE (glLoadMatrixf (cogl_matrix_get_array (matrix)) );
+      stack->flushed_identity = FALSE;
+    }
+}
+
+#endif /* HAVE_COGL_GLES2 */
+
+void
+_cogl_matrix_stack_prepare_for_flush (CoglMatrixStack *stack,
+                                      CoglMatrixMode mode,
+                                      CoglMatrixStackFlushFunc callback,
+                                      void *user_data)
+{
+  CoglMatrixState *state;
+
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
+
+  state = _cogl_matrix_stack_top (stack);
+
+  /* Because Cogl defines texture coordinates to have a top left origin and
+   * because offscreen framebuffers may be used for rendering to textures we
+   * always render upside down to offscreen buffers.
+   */
+  if (mode == COGL_MATRIX_PROJECTION &&
+      cogl_is_offscreen (_cogl_get_framebuffer ()))
+    {
+      CoglMatrix flipped_projection;
+      CoglMatrix *projection =
+        state->is_identity ? &ctx->identity_matrix : &state->matrix;
+
+      cogl_matrix_multiply (&flipped_projection,
+                            &ctx->y_flip_matrix, projection);
+      callback (FALSE, &flipped_projection, user_data);
+    }
+  else
+    callback (state->is_identity,
+              state->is_identity ? &ctx->identity_matrix : &state->matrix,
+              user_data);
 }
 
 void
@@ -382,6 +460,40 @@ _cogl_matrix_stack_flush_to_gl (CoglMatrixStack *stack,
 
   state = _cogl_matrix_stack_top (stack);
 
+#ifdef HAVE_COGL_GLES2
+
+  /* Under GLES2 we need to flush the matrices differently because
+     they are stored in uniforms attached to the program instead of
+     the global GL context state. At this point we can't be sure that
+     the right program will be generated so instead we'll just store a
+     reference to the matrix stack that is intended to be flushed and
+     update the uniform once the program is ready. */
+
+  switch (mode)
+    {
+    case COGL_MATRIX_MODELVIEW:
+      cogl_object_ref (stack);
+      if (ctx->flushed_modelview_stack)
+        cogl_object_unref (ctx->flushed_modelview_stack);
+      ctx->flushed_modelview_stack = stack;
+      break;
+
+    case COGL_MATRIX_PROJECTION:
+      cogl_object_ref (stack);
+      if (ctx->flushed_projection_stack)
+        cogl_object_unref (ctx->flushed_projection_stack);
+      ctx->flushed_projection_stack = stack;
+      break;
+
+    case COGL_MATRIX_TEXTURE:
+      /* This shouldn't happen because the texture matrices are
+         handled by the GLSL pipeline backend */
+      g_assert_not_reached ();
+      break;
+    }
+
+#else /* HAVE_COGL_GLES2 */
+
   if (stack->flushed_state == state)
     return;
 
@@ -408,36 +520,13 @@ _cogl_matrix_stack_flush_to_gl (CoglMatrixStack *stack,
       ctx->flushed_matrix_mode = mode;
     }
 
-  /* Because Cogl defines texture coordinates to have a top left origin and
-   * because offscreen framebuffers may be used for rendering to textures we
-   * always render upside down to offscreen buffers.
-   */
-  if (mode == COGL_MATRIX_PROJECTION &&
-      cogl_is_offscreen (_cogl_get_framebuffer ()))
-    {
-      CoglMatrix flipped_projection;
-      CoglMatrix *projection =
-        state->is_identity ? &ctx->identity_matrix : &state->matrix;
+  _cogl_matrix_stack_prepare_for_flush (stack,
+                                        mode,
+                                        flush_to_fixed_api_gl,
+                                        stack);
+
+#endif /* HAVE_COGL_GLES2 */
 
-      cogl_matrix_multiply (&flipped_projection,
-                            &ctx->y_flip_matrix, projection);
-      GE (glLoadMatrixf (cogl_matrix_get_array (&flipped_projection)));
-      stack->flushed_identity = FALSE;
-    }
-  else
-    {
-      if (state->is_identity)
-        {
-          if (!stack->flushed_identity)
-            GE (glLoadIdentity ());
-          stack->flushed_identity = TRUE;
-        }
-      else
-        {
-            GE (glLoadMatrixf (cogl_matrix_get_array (&state->matrix)));
-          stack->flushed_identity = FALSE;
-        }
-    }
   stack->flushed_state = state;
 }
 
@@ -448,3 +537,14 @@ _cogl_matrix_stack_dirty (CoglMatrixStack *stack)
   stack->flushed_identity = FALSE;
 }
 
+unsigned int
+_cogl_matrix_stack_get_age (CoglMatrixStack *stack)
+{
+  return stack->age;
+}
+
+gboolean
+_cogl_matrix_stack_has_identity_flag (CoglMatrixStack *stack)
+{
+  return _cogl_matrix_stack_top (stack)->is_identity;
+}
