@@ -28,20 +28,15 @@
 #include "clutter-glx.h"
 #include "clutter-profile.h"
 
-#include "../clutter-main.h"
-#include "../clutter-feature.h"
-#include "../clutter-color.h"
-#include "../clutter-util.h"
-#include "../clutter-event.h"
-#include "../clutter-enum-types.h"
-#include "../clutter-private.h"
-#include "../clutter-debug.h"
-#include "../clutter-units.h"
-#include "../clutter-shader.h"
-#include "../clutter-group.h"
-#include "../clutter-container.h"
-#include "../clutter-stage.h"
-#include "../clutter-stage-window.h"
+#include "clutter-actor-private.h"
+#include "clutter-debug.h"
+#include "clutter-device-manager.h"
+#include "clutter-event.h"
+#include "clutter-enum-types.h"
+#include "clutter-feature.h"
+#include "clutter-main.h"
+#include "clutter-private.h"
+#include "clutter-stage-private.h"
 
 #include "cogl/cogl.h"
 
@@ -57,15 +52,21 @@
 #include <drm.h>
 #endif
 
-static void clutter_stage_window_iface_init (ClutterStageWindowIface *iface);
+static void clutter_stage_window_iface_init     (ClutterStageWindowIface     *iface);
+static void clutter_event_translator_iface_init (ClutterEventTranslatorIface *iface);
 
-static ClutterStageWindowIface *clutter_stage_glx_parent_iface = NULL;
+static ClutterStageWindowIface *clutter_stage_window_parent_iface = NULL;
+static ClutterEventTranslatorIface *clutter_event_translator_parent_iface = NULL;
+
+#define clutter_stage_glx_get_type      _clutter_stage_glx_get_type
 
 G_DEFINE_TYPE_WITH_CODE (ClutterStageGLX,
-                         _clutter_stage_glx,
+                         clutter_stage_glx,
                          CLUTTER_TYPE_STAGE_X11,
                          G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_STAGE_WINDOW,
-                                                clutter_stage_window_iface_init));
+                                                clutter_stage_window_iface_init)
+                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_EVENT_TRANSLATOR,
+                                                clutter_event_translator_iface_init));
 
 static void
 clutter_stage_glx_unrealize (ClutterStageWindow *stage_window)
@@ -76,7 +77,7 @@ clutter_stage_glx_unrealize (ClutterStageWindow *stage_window)
   ClutterStageGLX *stage_glx = CLUTTER_STAGE_GLX (stage_window);
 
   /* Note unrealize should free up any backend stage related resources */
-  CLUTTER_NOTE (BACKEND, "Unrealizing stage");
+  CLUTTER_NOTE (BACKEND, "Unrealizing GLX stage [%p]", stage_glx);
 
   clutter_x11_trap_x_errors ();
 
@@ -86,19 +87,11 @@ clutter_stage_glx_unrealize (ClutterStageWindow *stage_window)
       stage_glx->glxwin = None;
     }
 
-  if (!stage_x11->is_foreign_xwin && stage_x11->xwin != None)
-    {
-      XDestroyWindow (backend_x11->xdpy, stage_x11->xwin);
-      stage_x11->xwin = None;
-    }
-  else
-    stage_x11->xwin = None;
+  _clutter_stage_x11_destroy_window_untrapped (stage_x11);
 
   XSync (backend_x11->xdpy, False);
 
   clutter_x11_untrap_x_errors ();
-
-  CLUTTER_MARK ();
 }
 
 static gboolean
@@ -109,7 +102,6 @@ clutter_stage_glx_realize (ClutterStageWindow *stage_window)
   ClutterBackendX11 *backend_x11;
   ClutterBackendGLX *backend_glx;
   ClutterBackend *backend;
-  int event_flags;
 
   CLUTTER_NOTE (ACTOR, "Realizing stage '%s' [%p]",
                 G_OBJECT_TYPE_NAME (stage_window),
@@ -119,62 +111,8 @@ clutter_stage_glx_realize (ClutterStageWindow *stage_window)
   backend_glx = CLUTTER_BACKEND_GLX (backend);
   backend_x11 = CLUTTER_BACKEND_X11 (backend);
 
-  if (stage_x11->xwin == None)
-    {
-      XSetWindowAttributes xattr;
-      unsigned long mask;
-      XVisualInfo *xvisinfo;
-      gfloat width, height;
-
-      CLUTTER_NOTE (MISC, "Creating stage X window");
-
-      xvisinfo = clutter_backend_x11_get_visual_info (backend_x11);
-      if (xvisinfo == NULL)
-        {
-          g_critical ("Unable to find suitable GL visual.");
-          return FALSE;
-        }
-
-      /* window attributes */
-      xattr.background_pixel = WhitePixel (backend_x11->xdpy,
-                                           backend_x11->xscreen_num);
-      xattr.border_pixel = 0;
-      xattr.colormap = XCreateColormap (backend_x11->xdpy,
-                                        backend_x11->xwin_root,
-                                        xvisinfo->visual,
-                                        AllocNone);
-      mask = CWBorderPixel | CWColormap;
-
-      /* Call get_size - this will either get the geometry size (which
-       * before we create the window is set to 640x480), or if a size
-       * is set, it will get that. This lets you set a size on the
-       * stage before it's realized.
-       */
-      clutter_actor_get_size (CLUTTER_ACTOR (stage_x11->wrapper),
-                              &width,
-                              &height);
-      stage_x11->xwin_width = (gint)width;
-      stage_x11->xwin_height = (gint)height;
-
-      stage_x11->xwin = XCreateWindow (backend_x11->xdpy,
-                                       backend_x11->xwin_root,
-                                       0, 0,
-                                       stage_x11->xwin_width,
-                                       stage_x11->xwin_height,
-                                       0,
-                                       xvisinfo->depth,
-                                       InputOutput,
-                                       xvisinfo->visual,
-                                       mask, &xattr);
-
-      CLUTTER_NOTE (BACKEND, "Stage [%p], window: 0x%x, size: %dx%d",
-                    stage_window,
-                    (unsigned int) stage_x11->xwin,
-                    stage_x11->xwin_width,
-                    stage_x11->xwin_height);
-
-      XFree (xvisinfo);
-    }
+  if (!_clutter_stage_x11_create_window (stage_x11))
+    return FALSE;
 
   if (stage_glx->glxwin == None)
     {
@@ -184,7 +122,8 @@ clutter_stage_glx_realize (ClutterStageWindow *stage_window)
 
       /* Try and create a GLXWindow to use with extensions dependent on
        * GLX versions >= 1.3 that don't accept regular X Windows as GLX
-       * drawables. */
+       * drawables.
+       */
       if (glXQueryVersion (backend_x11->xdpy, &major, &minor) &&
           major == 1 && minor >= 3 &&
           _clutter_backend_glx_get_fbconfig (backend_glx, &config))
@@ -196,56 +135,16 @@ clutter_stage_glx_realize (ClutterStageWindow *stage_window)
         }
     }
 
-  /* the masks for the events we want to select on a stage window;
-   * KeyPressMask and KeyReleaseMask are necessary even with XI1
-   * because key events are broken with that extension, and will
-   * be fixed by XI2
-   */
-  event_flags = StructureNotifyMask
-              | FocusChangeMask
-              | ExposureMask
-              | PropertyChangeMask
-              | EnterWindowMask
-              | LeaveWindowMask
-              | KeyPressMask
-              | KeyReleaseMask;
-
-  /* if we don't use XI1 then we also want core pointer events */
-  if (!clutter_x11_has_xinput ())
-    event_flags |= (ButtonPressMask | ButtonReleaseMask | PointerMotionMask);
-#ifdef HAVE_XINPUT
-  else
-    _clutter_x11_select_events (stage_x11->xwin);
-#endif
-
-  /* we unconditionally select input events even with event retrieval
-   * disabled because we need to guarantee that the Clutter internal
-   * state is maintained when calling clutter_x11_handle_event() without
-   * requiring applications or embedding toolkits to select events
-   * themselves. if we did that, we'd have to document the events to be
-   * selected, and also update applications and embedding toolkits each
-   * time we added a new mask, or a new class of events.
-   *
-   * see: http://bugzilla.clutter-project.org/show_bug.cgi?id=998
-   * for the rationale of why we did conditional selection. it is now
-   * clear that a compositor should clear out the input region, since
-   * it cannot assume a perfectly clean slate coming from us.
-   *
-   * see: http://bugzilla.clutter-project.org/show_bug.cgi?id=2228
-   * for an example of things that break if we do conditional event
-   * selection.
-   */
-  XSelectInput (backend_x11->xdpy, stage_x11->xwin, event_flags);
-
 #ifdef GLX_INTEL_swap_event
   if (clutter_feature_available (CLUTTER_FEATURE_SWAP_EVENTS))
     {
-      GLXDrawable drawable =
-        stage_glx->glxwin ? stage_glx->glxwin : stage_x11->xwin;
+      GLXDrawable drawable = stage_glx->glxwin
+                           ? stage_glx->glxwin
+                           : stage_x11->xwin;
 
-      /* similarly to above, we unconditionally select this event
-       * because we rely on it to advance the master clock, and
-       * drive redraw/relayout, animations and event handling.
+      /* we unconditionally select this event because we rely on it to
+       * advance the master clock, and drive redraw/relayout, animations
+       * and event handling.
        */
       glXSelectEvent (backend_x11->xdpy,
                       drawable,
@@ -253,16 +152,8 @@ clutter_stage_glx_realize (ClutterStageWindow *stage_window)
     }
 #endif /* GLX_INTEL_swap_event */
 
-  /* no user resize.. */
-  clutter_stage_x11_fix_window_size (stage_x11,
-                                     stage_x11->xwin_width,
-                                     stage_x11->xwin_height);
-  clutter_stage_x11_set_wm_protocols (stage_x11);
-
-  CLUTTER_NOTE (BACKEND, "Successfully realized stage");
-
   /* chain up to the StageX11 implementation */
-  return clutter_stage_glx_parent_iface->realize (stage_window);
+  return clutter_stage_window_parent_iface->realize (stage_window);
 }
 
 static int
@@ -274,21 +165,12 @@ clutter_stage_glx_get_pending_swaps (ClutterStageWindow *stage_window)
 }
 
 static void
-clutter_stage_glx_dispose (GObject *gobject)
+clutter_stage_glx_class_init (ClutterStageGLXClass *klass)
 {
-  G_OBJECT_CLASS (_clutter_stage_glx_parent_class)->dispose (gobject);
 }
 
 static void
-_clutter_stage_glx_class_init (ClutterStageGLXClass *klass)
-{
-  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-
-  gobject_class->dispose = clutter_stage_glx_dispose;
-}
-
-static void
-_clutter_stage_glx_init (ClutterStageGLX *stage)
+clutter_stage_glx_init (ClutterStageGLX *stage)
 {
 }
 
@@ -297,9 +179,16 @@ clutter_stage_glx_has_redraw_clips (ClutterStageWindow *stage_window)
 {
   ClutterStageGLX *stage_glx = CLUTTER_STAGE_GLX (stage_window);
 
-  /* NB: a degenerate clip means a full stage redraw is required */
-  if (stage_glx->initialized_redraw_clip &&
-      stage_glx->bounding_redraw_clip.width != 0)
+  /* NB: at the start of each new frame there is an implied clip that
+   * clips everything (i.e. nothing would be drawn) so we need to make
+   * sure we return True in the un-initialized case here.
+   *
+   * NB: a clip width of 0 means a full stage redraw has been queued
+   * so we effectively don't have any redraw clips in that case.
+   */
+  if (!stage_glx->initialized_redraw_clip ||
+      (stage_glx->initialized_redraw_clip &&
+       stage_glx->bounding_redraw_clip.width != 0))
     return TRUE;
   else
     return FALSE;
@@ -310,7 +199,7 @@ clutter_stage_glx_ignoring_redraw_clips (ClutterStageWindow *stage_window)
 {
   ClutterStageGLX *stage_glx = CLUTTER_STAGE_GLX (stage_window);
 
-  /* NB: a degenerate clip means a full stage redraw is required */
+  /* NB: a clip width of 0 means a full stage redraw is required */
   if (stage_glx->initialized_redraw_clip &&
       stage_glx->bounding_redraw_clip.width == 0)
     return TRUE;
@@ -375,7 +264,7 @@ clutter_stage_glx_add_redraw_clip (ClutterStageWindow *stage_window,
     return;
 
   /* A NULL stage clip means a full stage redraw has been queued and
-   * we keep track of this by setting a degenerate
+   * we keep track of this by setting a zero width
    * stage_glx->bounding_redraw_clip */
   if (stage_clip == NULL)
     {
@@ -414,7 +303,7 @@ clutter_stage_glx_add_redraw_clip (ClutterStageWindow *stage_window,
   if (redraw_area > (stage_area * 0.7f))
     {
       g_print ("DEBUG: clipped redraw too big, forcing full redraw\n");
-      /* Set a degenerate clip to force a full redraw */
+      /* Set a zero width clip to force a full redraw */
       stage_glx->bounding_redraw_clip.width = 0;
     }
 #endif
@@ -425,7 +314,7 @@ clutter_stage_glx_add_redraw_clip (ClutterStageWindow *stage_window,
 static void
 clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
 {
-  clutter_stage_glx_parent_iface = g_type_interface_peek_parent (iface);
+  clutter_stage_window_parent_iface = g_type_interface_peek_parent (iface);
 
   iface->realize = clutter_stage_glx_realize;
   iface->unrealize = clutter_stage_glx_unrealize;
@@ -438,6 +327,55 @@ clutter_stage_window_iface_init (ClutterStageWindowIface *iface)
   /* the rest is inherited from ClutterStageX11 */
 }
 
+static ClutterTranslateReturn
+clutter_stage_glx_translate_event (ClutterEventTranslator *translator,
+                                   gpointer                native,
+                                   ClutterEvent           *event)
+{
+#ifdef GLX_INTEL_swap_event
+  ClutterBackendGLX *backend_glx;
+  XEvent *xevent = native;
+
+  backend_glx = CLUTTER_BACKEND_GLX (clutter_get_default_backend ());
+
+  if (xevent->type == (backend_glx->event_base + GLX_BufferSwapComplete))
+    {
+      ClutterStageX11 *stage_x11 = CLUTTER_STAGE_X11 (translator);
+      ClutterStageGLX *stage_glx = CLUTTER_STAGE_GLX (translator);
+      GLXBufferSwapComplete *swap_complete_event;
+
+      swap_complete_event = (GLXBufferSwapComplete *) xevent;
+
+      if (stage_x11->xwin == swap_complete_event->drawable)
+        {
+	  /* Early versions of the swap_event implementation in Mesa
+	   * deliver BufferSwapComplete event when not selected for,
+	   * so if we get a swap event we aren't expecting, just ignore it.
+	   *
+	   * https://bugs.freedesktop.org/show_bug.cgi?id=27962
+	   */
+          if (stage_glx->pending_swaps > 0)
+            stage_glx->pending_swaps--;
+
+          return CLUTTER_TRANSLATE_REMOVE;
+        }
+    }
+#endif
+
+  /* chain up to the common X11 implementation */
+  return clutter_event_translator_parent_iface->translate_event (translator,
+                                                                 native,
+                                                                 event);
+}
+
+static void
+clutter_event_translator_iface_init (ClutterEventTranslatorIface *iface)
+{
+  clutter_event_translator_parent_iface = g_type_interface_peek_parent (iface);
+
+  iface->translate_event = clutter_stage_glx_translate_event;
+}
+
 #ifdef HAVE_DRM
 static int
 drm_wait_vblank(int fd, drm_wait_vblank_t *vbl)
@@ -490,7 +428,7 @@ wait_for_vblank (ClutterBackendGLX *backend_glx)
 
 void
 _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
-                          ClutterStage *stage)
+                           ClutterStage    *stage)
 {
   ClutterBackend    *backend;
   ClutterBackendX11 *backend_x11;
@@ -516,16 +454,18 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
                         "The time spent in _glx_blit_sub_buffer",
                         0 /* no application private data */);
 
+  stage_x11 = CLUTTER_STAGE_X11 (stage_glx);
+  if (stage_x11->xwin == None)
+    return;
+
   backend     = clutter_get_default_backend ();
   backend_x11 = CLUTTER_BACKEND_X11 (backend);
   backend_glx = CLUTTER_BACKEND_GLX (backend);
 
-  stage_x11 = CLUTTER_STAGE_X11 (stage_glx);
-
   CLUTTER_TIMER_START (_clutter_uprof_context, painting_timer);
 
   if (G_LIKELY (backend_glx->can_blit_sub_buffer) &&
-      /* NB: a degenerate redraw clip width == full stage redraw */
+      /* NB: a zero width redraw clip == full stage redraw */
       stage_glx->bounding_redraw_clip.width != 0 &&
       /* some drivers struggle to get going and produce some junk
        * frames when starting up... */
@@ -534,7 +474,9 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
        * artefacts. See clutter-event-x11.c:event_translate for a
        * detailed explanation */
       G_LIKELY (stage_x11->clipped_redraws_cool_off == 0))
-    may_use_clipped_redraw = TRUE;
+    {
+      may_use_clipped_redraw = TRUE;
+    }
   else
     may_use_clipped_redraw = FALSE;
 
@@ -557,7 +499,7 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
   else
     _clutter_stage_do_paint (stage, NULL);
 
-  if (clutter_paint_debug_flags & CLUTTER_DEBUG_REDRAWS &&
+  if ((clutter_paint_debug_flags & CLUTTER_DEBUG_REDRAWS) &&
       may_use_clipped_redraw)
     {
       ClutterGeometry *clip = &stage_glx->bounding_redraw_clip;
@@ -606,10 +548,9 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
   cogl_flush ();
   CLUTTER_TIMER_STOP (_clutter_uprof_context, painting_timer);
 
-  if (stage_x11->xwin == None)
-    return;
-
-  drawable = stage_glx->glxwin ? stage_glx->glxwin : stage_x11->xwin;
+  drawable = stage_glx->glxwin
+           ? stage_glx->glxwin
+           : stage_x11->xwin;
 
   /* If we might ever use _clutter_backend_glx_blit_sub_buffer then we
    * always need to keep track of the video_sync_count so that we can
@@ -646,7 +587,8 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
        * anyway so it should only exhibit temporary artefacts.
        */
       copy_area.y = clutter_actor_get_height (CLUTTER_ACTOR (stage))
-        - clip->y - clip->height;
+                  - clip->y
+                  - clip->height;
       copy_area.x = clip->x;
       copy_area.width = clip->width;
       copy_area.height = clip->height;
@@ -736,6 +678,7 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
       CLUTTER_TIMER_START (_clutter_uprof_context, swapbuffers_timer);
       glXSwapBuffers (backend_x11->xdpy, drawable);
       CLUTTER_TIMER_STOP (_clutter_uprof_context, swapbuffers_timer);
+      _cogl_swap_buffers_notify ();
     }
 
   backend_glx->last_video_sync_count = video_sync_count;
@@ -745,4 +688,3 @@ _clutter_stage_glx_redraw (ClutterStageGLX *stage_glx,
 
   stage_glx->frame_count++;
 }
-
